@@ -237,15 +237,23 @@ class Project(Model):
              projectIri_SName: IriOrNCName | Iri | Xsd_NCName | str,
              ignore_cache: bool = False) -> Self:
         """
-        Read the project from the triplestore and return an instance of the project
-        :param con: A valid Connection object
+        Reads the project from the triplestore and returns an instance of the project.
+
+        This method queries the triplestore based on a provided IRI or shortname of the project,
+        optionally utilizing a cache for optimization. If the project is found, it fetches and
+        processes the project's data including metadata and creates an instance of the project.
+        If not found, an appropriate error is raised.
+
+        :param con: A valid connection to the triplestore.
         :type con: IConnection
-        :param projectIri_SName: The IRI or shortname of the project to be read
-        :type projectIri_SName: Iri | Xsd_NCName | str
-        :return: Project instance
-        :rtype: Project
-        :raise: OldapErrorNotFound: project with given Iri not found
-        :raise: OldapError: All other errors/problems
+        :param projectIri_SName: The IRI or shortname of the project to be read.
+        :type projectIri_SName: IriOrNCName | Iri | Xsd_NCName | str
+        :param ignore_cache: Whether to bypass the cache and query the triplestore directly.
+        :type ignore_cache: bool
+        :return: An instance of the project with the fetched data.
+        :rtype: Self
+        :raises OldapErrorNotFound: If the project with the specified IRI or shortname is not found.
+        :raises OldapError: If other errors or problems occur during the process.
         """
         context = Context(name=con.context_name)
         query = context.sparql_context
@@ -363,22 +371,23 @@ class Project(Model):
     def search(con: IConnection,
                label: Xsd_string | str | None = None,
                comment: Xsd_string | str | None = None) -> list[ProjectSearchResult]:
-
-
         """
-        Search for a given project. If no label or comment is given, all existing projects are returned. If both
-        a search term for the label and comment are given, they will be combined by *AND*.
-        :param con: Valid Connection object
+        Search for projects based on the provided label and/or comment. If no label or comment is provided, all
+        existing projects are returned. When both label and comment are specified, the search combines their
+        conditions using logical AND.
+
+        :param con: Valid connection object representing the current context
         :type con: IConnection
-        :param label: A string to search for in the labels. The search will check if the label of a project
-        **contains** the string given here.
-        :type label: str
-        :param comment: A string to search for in the comments. The search will check if the comment of a project
-        **contains** the string given here
-        :type comment: str
-        :return: List of IRIs and shortnames matching the search criteria (AnyIRI | QName)
+        :param label: String to search within project labels. The label search is performed by checking
+            if a project's label contains the specified string.
+        :type label: Xsd_string or str or None
+        :param comment: String to search within project comments. The comment search is performed by
+            checking if a project's comment contains the specified string.
+        :type comment: Xsd_string or str or None
+        :return: List of project search results matching the specified criteria.
         :rtype: list[ProjectSearchResult]
-        :raises OldapErrorNotFound: If the project does not exist
+        :raises OldapErrorNotFound: Raised if the project does not exist in the repository or cannot
+            be located based on the search criteria.
         """
         label = Xsd_string(label, validate=True)
         comment = Xsd_string(comment, validate=True)
@@ -413,14 +422,25 @@ class Project(Model):
 
     def create(self, indent: int = 0, indent_inc: int = 4) -> None:
         """
-        Create a new project in the triple store
-        :param indent: Start indent level for generated SPARQL (debugging)
+        Creates a new project in the triple store. The function starts by
+        checking if the connection is available, verifies if the user
+        has the required permissions, and then proceeds to execute
+        SPARQL queries to ensure the project does not already exist.
+        If the project doesn't exist, it creates the project by
+        constructing and executing SPARQL INSERT queries. It also
+        updates the project metadata, including timestamps and user
+        information.
+
+        :param indent: The start indent level for the generated SPARQL
+            query, primarily used for debugging.
         :type indent: int
-        :param indent_inc: Indent increment
+        :param indent_inc: The increment for indentation level, which
+            affects how the SPARQL query is formatted.
         :type indent_inc: int
         :return: None
-        :raises OldapErrorAlreadyExists: If a project with the projectIri already exists
-        :raises OldapError: All other errors
+        :raises OldapErrorAlreadyExists: If a project already exists with
+            the same project IRI.
+        :raises OldapError: If any other errors occur during execution.
         """
         if self._con is None:
             raise OldapError("Cannot create: no connection")
@@ -484,15 +504,21 @@ class Project(Model):
 
     def update(self, indent: int = 0, indent_inc: int = 4) -> None:
         """
-        Write all the modifications that were applied to the project instqnce to the triple store.
-        :param indent: Starting indent for SPARQL queries [Only used for debbugging purposes]
+        Updates all modifications made to the project instance and writes them to the triple store.
+
+        The function ensures changes contained within the project instance are written to the triple store
+        by serializing them into SPARQL queries and executing them. Permissions are checked before
+        execution, and transactions are used to ensure atomicity. If the update fails, appropriate
+        exceptions are raised, and operations are rolled back.
+
+        :param indent: Starting indentation level used for SPARQL query formatting.
         :type indent: int
-        :param indent_inc: Indent increment for SPARQL queries [Only used for debbugging purposes]
+        :param indent_inc: Increment of indentation for formatting SPARQL queries.
         :type indent_inc: int
         :return: None
-        :Raises: OldapErrorNoPermission: No permission for operation
-        :Raises: OldapErrorUpdateFailed: Update failed
-        :Raises: Oldap Error: Other Internal error
+        :raises OldapErrorNoPermission: Raised if the user does not have permissions for the operation.
+        :raises OldapErrorUpdateFailed: Raised if the update fails due to timestamp mismatch or other inconsistencies.
+        :raises OldapError: Raised for other internal errors.
         """
         result, message = self.check_for_permissions()
         if not result:
@@ -558,10 +584,14 @@ class Project(Model):
 
     def delete(self) -> None:
         """
-        Delete the given user from the triplestore
+        Delete the specified user from the triplestore. The function ensures that the
+        user has the necessary permissions before proceeding with the deletion process.
+        Additionally, it cleans up related cache entries.
+
+        :raises OldapErrorNoPermission: Raised if the user lacks the necessary
+            permissions to perform the operation.
+        :raises OldapError: Raised in case of generic internal errors.
         :return: None
-        :raises OldapErrorNoPermission: No permission for operation
-        :raises OldapError: generic internal error
         """
         result, message = self.check_for_permissions()
         if not result:
@@ -587,6 +617,22 @@ class Project(Model):
 
     @staticmethod
     def get_shortname_from_iri(con: IConnection, iri: Iri) -> Xsd_NCName:
+        """
+        Extracts the project short name from the given IRI using a SPARQL query executed
+        in the provided connection. The method communicates with the server to fetch
+        the short name associated with the specified IRI. If the IRI does not resolve
+        to exactly one short name, an exception is raised.
+
+        :param con: The connection object used to interact with the data source.
+        :type con: IConnection
+        :param iri: An instance of Iri or a value representing the IRI to query.
+        :type iri: Iri
+        :return: The project short name associated with the provided IRI.
+        :rtype: Xsd_NCName
+        :raises OldapError: If the connection object is not an instance of IConnection.
+        :raises OldapErrorNotFound: If no short name is found or if multiple results are
+            returned for the specified IRI.
+        """
         if not isinstance(con, IConnection):
             raise OldapError("Connection must be an instance of IConnection")
         if not isinstance(iri, Iri):
